/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include "cpci405.h"
#include <asm/processor.h>
#include <405gp_i2c.h>

/* ------------------------------------------------------------------------- */

#if 0
#define FPGA_DEBUG
#endif

#define FPGA_PRG_SLEEP         32        /* fpga program sleep-time */

#define IBM405GP_GPIO0_OR      0xef600700  /* GPIO Output */
#define IBM405GP_GPIO0_TCR     0xef600704  /* GPIO Three-State Control */
#define IBM405GP_GPIO0_ODR     0xef600718  /* GPIO Open Drain */
#define IBM405GP_GPIO0_IR      0xef60071c  /* GPIO Input */

/* fpga configuration data - generated by bit2inc */
const unsigned char fpgadata[] =
{
#include "fpgadata.c"
};


/******************************************************************************
 *
 * fpga_boot - Load fpga-image into fpga
 *
 */
static int fpga_boot(void)
{
  int i,j,index,len;
  unsigned char b;
  int size = sizeof(fpgadata);

  /* display infos on fpgaimage */
  index = 15;
  for (i=0; i<4; i++)
    {
      len = fpgadata[index];
#ifdef FPGA_DEBUG
      printf("FPGA: %s\n", &(fpgadata[index+1]));
#endif
      index += len+3;
    }

  /* search for preamble 0xFF2X */
  for (index = 0; index < size-1 ; index++)
    {
      if ((fpgadata[index] == 0xff) && ((fpgadata[index+1] & 0xf0) == 0x20))
	break;
    }
#ifdef FPGA_DEBUG
  printf("FPGA: configdata starts at position 0x%x\n",index);
  printf("FPGA: length of fpga-data %d\n", size-index);
#endif

  /*
   * Setup port pins for fpga programming
   */
  out32(IBM405GP_GPIO0_ODR, 0x00000000);  /* no open drain pins */
  out32(IBM405GP_GPIO0_TCR, 0x07000000);  /* setup for output */
  out32(IBM405GP_GPIO0_OR, 0x07000000);   /* set output pins to high (default) */

#ifdef FPGA_DEBUG
  printf("%s\n",((in32(IBM405GP_GPIO0_IR) & 0x00800000) == 0) ? "NOT DONE" : "DONE" );
#endif

  /*
   * Init fpga by asserting and deasserting PROGRAM*
   */
  out32(IBM405GP_GPIO0_OR, 0x03000000);
  udelay(FPGA_PRG_SLEEP*1000);
#ifdef FPGA_DEBUG
  printf("%s\n",((in32(IBM405GP_GPIO0_IR) & 0x00800000) == 0) ? "NOT DONE" : "DONE" );
#endif
  out32(IBM405GP_GPIO0_OR, 0x07000000);
  udelay(FPGA_PRG_SLEEP*1000);
#ifdef FPGA_DEBUG
  printf("%s\n",((in32(IBM405GP_GPIO0_IR) & 0x00800000) == 0) ? "NOT DONE" : "DONE" );
#endif

  /* write configuration-data into fpga... */
  for (i=index; i<size; i++)
    {
      b = fpgadata[i];
      for (j=0; j<8; j++)
        {
          if ((b & 0x80) == 0x80)
            {
              /*
               * Write 1 to fpga
               */
              out32(IBM405GP_GPIO0_OR, 0x05000000);  /* set clock to 0 */
              out32(IBM405GP_GPIO0_OR, 0x05000000);  /* set data to 1 */
              out32(IBM405GP_GPIO0_OR, 0x07000000);  /* set clock to 1 */
              out32(IBM405GP_GPIO0_OR, 0x07000000);  /* set data to 1 */
            }
          else
            {
              /*
               * Write 0 to fpga
               */
              out32(IBM405GP_GPIO0_OR, 0x05000000);  /* set clock to 0 */
              out32(IBM405GP_GPIO0_OR, 0x04000000);  /* set data to 0 */
              out32(IBM405GP_GPIO0_OR, 0x06000000);  /* set clock to 1 */
              out32(IBM405GP_GPIO0_OR, 0x07000000);  /* set data to 1 */
            }

          /*
           * Shift to next bit
           */
          b <<= 1;
        }
    }

  udelay(FPGA_PRG_SLEEP*1000);
#ifdef FPGA_DEBUG
  printf("%s\n",((in32(IBM405GP_GPIO0_IR) & 0x00800000) == 0) ? "NOT DONE" : "DONE" );
#endif

  /*
   * Check if fpga's DONE signal - correctly booted ?
   */
  if ((in32(IBM405GP_GPIO0_IR) & 0x00800000) == 0)
    {
#ifdef FPGA_DEBUG
      printf("FPGA: Booting failed!\n");
#endif
      return -1;
    }
  else
    {
#ifdef FPGA_DEBUG
      printf("FPGA: Booting successful!\n");
#endif
      return 0;
    }
}


int board_pre_init (void)
{
  /*
   * Boot onboard FPGA
   */
  while (fpga_boot() != 0);

  /*
   * IRQ 0-15  405GP internally generated; active high; level sensitive
   * IRQ 16    405GP internally generated; active low; level sensitive
   * IRQ 17-24 RESERVED
   * IRQ 25 (EXT IRQ 0) CAN0; active low; level sensitive
   * IRQ 26 (EXT IRQ 1) CAN1; active low; level sensitive
   * IRQ 27 (EXT IRQ 2) PCI SLOT 0; active low; level sensitive
   * IRQ 28 (EXT IRQ 3) PCI SLOT 1; active low; level sensitive
   * IRQ 29 (EXT IRQ 4) PCI SLOT 2; active low; level sensitive
   * IRQ 30 (EXT IRQ 5) PCI SLOT 3; active low; level sensitive
   * IRQ 31 (EXT IRQ 6) COMPACT FLASH; active high; level sensitive
   */
  mtdcr(uicsr, 0xFFFFFFFF);        /* clear all ints */
  mtdcr(uicer, 0x00000000);        /* disable all ints */
  mtdcr(uiccr, 0x00000000);        /* set all to be non-critical*/
  mtdcr(uicpr, 0xFFFFFF81);        /* set int polarities */
  mtdcr(uictr, 0x10000000);        /* set int trigger levels */
  mtdcr(uicvcr, 0x00000001);       /* set vect base=0,INT0 highest priority*/
  mtdcr(uicsr, 0xFFFFFFFF);        /* clear all ints */

  return 0;
}


/* ------------------------------------------------------------------------- */

int ctermm2(void)
{
  if ((*(unsigned char *)0xf0000400 == 0x00) &&
      (*(unsigned char *)0xf0000401 == 0x01))
    return 0;                           /* no, board is cpci405 */
  else
    return -1;                          /* yes, board is cterm-m2 */
}


int misc_init_f(void)
{
  return 0;  /* dummy implementation */
}


void misc_init_r(bd_t *bd)
{
  /*
   * Generate last byte of ip-addr from code-plug @ 0xf0000400
   */
  if (ctermm2())
    {
      char str[32];
      unsigned char ipbyte = *(unsigned char *)0xf0000400;

      /*
       * Only overwrite ip-addr with allowed values
       */
      if ((ipbyte != 0x00) && (ipbyte != 0xff))
        {
          bd->bi_ip_addr = (bd->bi_ip_addr & 0xffffff00) | ipbyte;
          sprintf(str, "%ld.%ld.%ld.%ld",
                  (bd->bi_ip_addr & 0xff000000) >> 24,
                  (bd->bi_ip_addr & 0x00ff0000) >> 16,
                  (bd->bi_ip_addr & 0x0000ff00) >> 8,
                  (bd->bi_ip_addr & 0x000000ff));
          setenv("ipaddr", str);
        }
    }
}


/*
 * Check Board Identity:
 */

int checkboard (void)
{
    unsigned char *s = getenv("serial#");
    unsigned char *e;
    int i;
    int index;
    int len;

    if (!s)
      {
	printf ("### No HW ID - assuming CPCI405");
      }
    else
      {
        for (e=s; *e; ++e) {
          if (*e == ' ')
	    break;
        }

        for ( ; s<e; ++s) {
          putc (*s);
        }
      }

    if (ctermm2())
      printf(" (CTERM-M2 - Id=0x%02x)", *(unsigned char *)0xf0000400);

    printf("\n  FPGA:  ");

    /* display infos on fpgaimage */
    index = 15;
    for (i=0; i<4; i++)
      {
        len = fpgadata[index];
        printf("%s ", &(fpgadata[index+1]));
        index += len+3;
      }

    putc ('\n');

    return (0);
}

/* ------------------------------------------------------------------------- */

long int initdram (int board_type)
{
  return (16 * 1024*1024);
}

/* ------------------------------------------------------------------------- */

int testdram (void)
{
    /* TODO: XXX XXX XXX */
    printf ("test: 16 MB - ok\n");

    return (0);
}

/* ------------------------------------------------------------------------- */
